#include "atlas_misc.h"
#ifndef PFYDIST
   #define PFYDIST 4
#endif
#ifndef PFXDIST
   #define PFXDIST 0
#endif
#ifndef PFADIST
   #define PFADIST 0
#endif

#if !defined(ATL_3DNow) && !defined(ATL_SSE1) && \
    (PFADIST != 0 || PFXDIST != 0 || PFYDIST != 0)
   #include "atlas_prefetch.h"
#endif

#if defined(ATL_3DNow) || defined(ATL_SSE1)
   #ifndef PFIY
      #define PFIY prefetchnta
   #endif
   #ifndef PFIX
      #define PFIX prefetchnta
   #endif
   #ifndef PFIA
      #ifdef ATL_3DNow
         #define PFIA prefetchw
      #else
         #define PFIA prefetcht0
      #endif
   #endif
#endif
/*
 * X & A are prefetched in M loop PF[A,X]DIST (in bytes) ahead
 */
#if PFADIST == 0                /* flag for no prefetch */
   #define prefA(mem)
#else
   #if defined(ATL_3DNow) || defined(ATL_SSE1)
      #define prefA(mem) __asm__ __volatile__ \
         (Mstr(PFIA) " %0" : : "m" (*(((char *)(mem))+PFADIST)))
   #else
      #if PFLVL == 2
         #define prefA(mem) ATL_pfl2W(((char*)mem)+PFADIST)
      #else
         #define prefA(mem) ATL_pfl1W(((char*)mem)+PFADIST)
      #endif
   #endif
#endif
#if PFXDIST == 0                /* flag for no prefetch */
   #define prefX(mem)
#else
   #if defined(ATL_3DNow) || defined(ATL_SSE1)
      #define prefX(mem) __asm__ __volatile__ \
         (Mstr(PFIX) " %0" : : "m" (*(((char *)(mem))+PFXDIST)))
   #else
      #if PFLVL == 2
         #define prefX(mem) ATL_pfl2R(((char*)mem)+PFXDIST)
      #else
         #define prefX(mem) ATL_pfl1R(((char*)mem)+PFXDIST)
      #endif
   #endif
#endif
/*
 * Y is prefetched in N-loop, and always fetches next NU piece
 */
#if PFYDIST == 0                /* flag for no prefetch */
   #define prefY(mem)
#else
   #if defined(ATL_3DNow) || defined(ATL_SSE1)
      #define prefY(mem) \
         __asm__ __volatile__ (Mstr(PFIY) " %0" : : "m" (*(((char *)(mem)))))
   #else
      #if PFLVL == 2
         #define prefY(mem) ATL_pfl2R(mem)
      #else
         #define prefY(mem) ATL_pfl1R(mem)
      #endif
   #endif
#endif
#ifndef ATL_CINT
   #define ATL_CINT const int
#endif
#ifndef ATL_INT
   #define ATL_INT int
#endif
/* Need to handle BETA=0 case by assigning y to zero outside of loop */
void ATL_UGEMV(ATL_CINT M, ATL_CINT N, const TYPE *A, ATL_CINT lda,
               const TYPE *X, TYPE *Y)
/*
 * 32x4 unrolled mvn_c.
 * Extracted from ATLAS/tune/blas/gemv/atlas-l2g.base
 */
{
   ATL_CINT N4=(N/4)*4, M32=(M/32)*32, lda4=lda*4;
   ATL_INT j;

   #ifdef BETA0
      for (j=0; j < M; j++)
         Y[j] = ATL_rzero;
   #endif
   for (j=N4; j; j -= 4, A += lda4, X += 4)
   {
      const float *A0=A, *A1=A0+lda, *A2=A1+lda, *A3=A2+lda;
      const register float x0=X[0], x1=X[1], x2=X[2], x3=X[3];
      ATL_INT i;
      prefY(Y+4+4-1);
      for (i=0; i < M32; i += 32)
      {
         const float a0_0=A0[i+0], a1_0=A0[i+1], a2_0=A0[i+2], a3_0=A0[i+3],
                     a4_0=A0[i+4], a5_0=A0[i+5], a6_0=A0[i+6], a7_0=A0[i+7],
                     a8_0=A0[i+8], a9_0=A0[i+9], a10_0=A0[i+10], a11_0=A0[i+11],
                     a12_0=A0[i+12], a13_0=A0[i+13], a14_0=A0[i+14],
                     a15_0=A0[i+15], a16_0=A0[i+16], a17_0=A0[i+17],
                     a18_0=A0[i+18], a19_0=A0[i+19], a20_0=A0[i+20],
                     a21_0=A0[i+21], a22_0=A0[i+22], a23_0=A0[i+23],
                     a24_0=A0[i+24], a25_0=A0[i+25], a26_0=A0[i+26],
                     a27_0=A0[i+27], a28_0=A0[i+28], a29_0=A0[i+29],
                     a30_0=A0[i+30], a31_0=A0[i+31], a0_1=A1[i+0], a1_1=A1[i+1],
                     a2_1=A1[i+2], a3_1=A1[i+3], a4_1=A1[i+4], a5_1=A1[i+5],
                     a6_1=A1[i+6], a7_1=A1[i+7], a8_1=A1[i+8], a9_1=A1[i+9],
                     a10_1=A1[i+10], a11_1=A1[i+11], a12_1=A1[i+12],
                     a13_1=A1[i+13], a14_1=A1[i+14], a15_1=A1[i+15],
                     a16_1=A1[i+16], a17_1=A1[i+17], a18_1=A1[i+18],
                     a19_1=A1[i+19], a20_1=A1[i+20], a21_1=A1[i+21],
                     a22_1=A1[i+22], a23_1=A1[i+23], a24_1=A1[i+24],
                     a25_1=A1[i+25], a26_1=A1[i+26], a27_1=A1[i+27],
                     a28_1=A1[i+28], a29_1=A1[i+29], a30_1=A1[i+30],
                     a31_1=A1[i+31], a0_2=A2[i+0], a1_2=A2[i+1], a2_2=A2[i+2],
                     a3_2=A2[i+3], a4_2=A2[i+4], a5_2=A2[i+5], a6_2=A2[i+6],
                     a7_2=A2[i+7], a8_2=A2[i+8], a9_2=A2[i+9], a10_2=A2[i+10],
                     a11_2=A2[i+11], a12_2=A2[i+12], a13_2=A2[i+13],
                     a14_2=A2[i+14], a15_2=A2[i+15], a16_2=A2[i+16],
                     a17_2=A2[i+17], a18_2=A2[i+18], a19_2=A2[i+19],
                     a20_2=A2[i+20], a21_2=A2[i+21], a22_2=A2[i+22],
                     a23_2=A2[i+23], a24_2=A2[i+24], a25_2=A2[i+25],
                     a26_2=A2[i+26], a27_2=A2[i+27], a28_2=A2[i+28],
                     a29_2=A2[i+29], a30_2=A2[i+30], a31_2=A2[i+31],
                     a0_3=A3[i+0], a1_3=A3[i+1], a2_3=A3[i+2], a3_3=A3[i+3],
                     a4_3=A3[i+4], a5_3=A3[i+5], a6_3=A3[i+6], a7_3=A3[i+7],
                     a8_3=A3[i+8], a9_3=A3[i+9], a10_3=A3[i+10], a11_3=A3[i+11],
                     a12_3=A3[i+12], a13_3=A3[i+13], a14_3=A3[i+14],
                     a15_3=A3[i+15], a16_3=A3[i+16], a17_3=A3[i+17],
                     a18_3=A3[i+18], a19_3=A3[i+19], a20_3=A3[i+20],
                     a21_3=A3[i+21], a22_3=A3[i+22], a23_3=A3[i+23],
                     a24_3=A3[i+24], a25_3=A3[i+25], a26_3=A3[i+26],
                     a27_3=A3[i+27], a28_3=A3[i+28], a29_3=A3[i+29],
                     a30_3=A3[i+30], a31_3=A3[i+31];
         register float y0=Y[i+0], y1=Y[i+1], y2=Y[i+2], y3=Y[i+3], y4=Y[i+4],
                        y5=Y[i+5], y6=Y[i+6], y7=Y[i+7], y8=Y[i+8], y9=Y[i+9],
                        y10=Y[i+10], y11=Y[i+11], y12=Y[i+12], y13=Y[i+13],
                        y14=Y[i+14], y15=Y[i+15], y16=Y[i+16], y17=Y[i+17],
                        y18=Y[i+18], y19=Y[i+19], y20=Y[i+20], y21=Y[i+21],
                        y22=Y[i+22], y23=Y[i+23], y24=Y[i+24], y25=Y[i+25],
                        y26=Y[i+26], y27=Y[i+27], y28=Y[i+28], y29=Y[i+29],
                        y30=Y[i+30], y31=Y[i+31];

         prefX(X);
         y0 += a0_0 * x0;
         prefA(A0);
         y1 += a1_0 * x0;
         y2 += a2_0 * x0;
         y3 += a3_0 * x0;
         y4 += a4_0 * x0;
         y5 += a5_0 * x0;
         y6 += a6_0 * x0;
         y7 += a7_0 * x0;
         y8 += a8_0 * x0;
         y9 += a9_0 * x0;
         y10 += a10_0 * x0;
         y11 += a11_0 * x0;
         y12 += a12_0 * x0;
         y13 += a13_0 * x0;
         y14 += a14_0 * x0;
         y15 += a15_0 * x0;
         y16 += a16_0 * x0;
         prefA(A0);
         y17 += a17_0 * x0;
         y18 += a18_0 * x0;
         y19 += a19_0 * x0;
         y20 += a20_0 * x0;
         y21 += a21_0 * x0;
         y22 += a22_0 * x0;
         y23 += a23_0 * x0;
         y24 += a24_0 * x0;
         y25 += a25_0 * x0;
         y26 += a26_0 * x0;
         y27 += a27_0 * x0;
         y28 += a28_0 * x0;
         y29 += a29_0 * x0;
         y30 += a30_0 * x0;
         y31 += a31_0 * x0;
         y0 += a0_1 * x1;
         prefA(A1);
         y1 += a1_1 * x1;
         y2 += a2_1 * x1;
         y3 += a3_1 * x1;
         y4 += a4_1 * x1;
         y5 += a5_1 * x1;
         y6 += a6_1 * x1;
         y7 += a7_1 * x1;
         y8 += a8_1 * x1;
         y9 += a9_1 * x1;
         y10 += a10_1 * x1;
         y11 += a11_1 * x1;
         y12 += a12_1 * x1;
         y13 += a13_1 * x1;
         y14 += a14_1 * x1;
         y15 += a15_1 * x1;
         y16 += a16_1 * x1;
         prefA(A1);
         y17 += a17_1 * x1;
         y18 += a18_1 * x1;
         y19 += a19_1 * x1;
         y20 += a20_1 * x1;
         y21 += a21_1 * x1;
         y22 += a22_1 * x1;
         y23 += a23_1 * x1;
         y24 += a24_1 * x1;
         y25 += a25_1 * x1;
         y26 += a26_1 * x1;
         y27 += a27_1 * x1;
         y28 += a28_1 * x1;
         y29 += a29_1 * x1;
         y30 += a30_1 * x1;
         y31 += a31_1 * x1;
         y0 += a0_2 * x2;
         prefA(A2);
         y1 += a1_2 * x2;
         y2 += a2_2 * x2;
         y3 += a3_2 * x2;
         y4 += a4_2 * x2;
         y5 += a5_2 * x2;
         y6 += a6_2 * x2;
         y7 += a7_2 * x2;
         y8 += a8_2 * x2;
         y9 += a9_2 * x2;
         y10 += a10_2 * x2;
         y11 += a11_2 * x2;
         y12 += a12_2 * x2;
         y13 += a13_2 * x2;
         y14 += a14_2 * x2;
         y15 += a15_2 * x2;
         y16 += a16_2 * x2;
         prefA(A2);
         y17 += a17_2 * x2;
         y18 += a18_2 * x2;
         y19 += a19_2 * x2;
         y20 += a20_2 * x2;
         y21 += a21_2 * x2;
         y22 += a22_2 * x2;
         y23 += a23_2 * x2;
         y24 += a24_2 * x2;
         y25 += a25_2 * x2;
         y26 += a26_2 * x2;
         y27 += a27_2 * x2;
         y28 += a28_2 * x2;
         y29 += a29_2 * x2;
         y30 += a30_2 * x2;
         y31 += a31_2 * x2;
         y0 += a0_3 * x3;
         prefA(A3);
         y1 += a1_3 * x3;
         y2 += a2_3 * x3;
         y3 += a3_3 * x3;
         y4 += a4_3 * x3;
         y5 += a5_3 * x3;
         y6 += a6_3 * x3;
         y7 += a7_3 * x3;
         y8 += a8_3 * x3;
         y9 += a9_3 * x3;
         y10 += a10_3 * x3;
         y11 += a11_3 * x3;
         y12 += a12_3 * x3;
         y13 += a13_3 * x3;
         y14 += a14_3 * x3;
         y15 += a15_3 * x3;
         y16 += a16_3 * x3;
         prefA(A3);
         y17 += a17_3 * x3;
         y18 += a18_3 * x3;
         y19 += a19_3 * x3;
         y20 += a20_3 * x3;
         y21 += a21_3 * x3;
         y22 += a22_3 * x3;
         y23 += a23_3 * x3;
         y24 += a24_3 * x3;
         y25 += a25_3 * x3;
         y26 += a26_3 * x3;
         y27 += a27_3 * x3;
         y28 += a28_3 * x3;
         y29 += a29_3 * x3;
         y30 += a30_3 * x3;
         y31 += a31_3 * x3;
         Y[i+0] = y0;
         Y[i+1] = y1;
         Y[i+2] = y2;
         Y[i+3] = y3;
         Y[i+4] = y4;
         Y[i+5] = y5;
         Y[i+6] = y6;
         Y[i+7] = y7;
         Y[i+8] = y8;
         Y[i+9] = y9;
         Y[i+10] = y10;
         Y[i+11] = y11;
         Y[i+12] = y12;
         Y[i+13] = y13;
         Y[i+14] = y14;
         Y[i+15] = y15;
         Y[i+16] = y16;
         Y[i+17] = y17;
         Y[i+18] = y18;
         Y[i+19] = y19;
         Y[i+20] = y20;
         Y[i+21] = y21;
         Y[i+22] = y22;
         Y[i+23] = y23;
         Y[i+24] = y24;
         Y[i+25] = y25;
         Y[i+26] = y26;
         Y[i+27] = y27;
         Y[i+28] = y28;
         Y[i+29] = y29;
         Y[i+30] = y30;
         Y[i+31] = y31;
      }
      for (i=M32; i < M; i++)
      {
         const float a0_0=A0[i], a0_1=A1[i], a0_2=A2[i], a0_3=A3[i];
         register float y0 = Y[i];

         y0 += a0_0 * x0;
         y0 += a0_1 * x1;
         y0 += a0_2 * x2;
         y0 += a0_3 * x3;
         Y[i] = y0;
      }
   }
/*
 * Do remaining columns with NU=1 cleanup
 */
   for (j=N-N4; j; j--, A += lda, X++)
   {
      const float *A0=A;
      const register float x0=X[0];
      ATL_INT i;
      prefY(Y+1+1-1);
      for (i=0; i < M32; i += 32)
      {
         const float a0_0=A0[i+0], a1_0=A0[i+1], a2_0=A0[i+2], a3_0=A0[i+3],
                     a4_0=A0[i+4], a5_0=A0[i+5], a6_0=A0[i+6], a7_0=A0[i+7],
                     a8_0=A0[i+8], a9_0=A0[i+9], a10_0=A0[i+10], a11_0=A0[i+11],
                     a12_0=A0[i+12], a13_0=A0[i+13], a14_0=A0[i+14],
                     a15_0=A0[i+15], a16_0=A0[i+16], a17_0=A0[i+17],
                     a18_0=A0[i+18], a19_0=A0[i+19], a20_0=A0[i+20],
                     a21_0=A0[i+21], a22_0=A0[i+22], a23_0=A0[i+23],
                     a24_0=A0[i+24], a25_0=A0[i+25], a26_0=A0[i+26],
                     a27_0=A0[i+27], a28_0=A0[i+28], a29_0=A0[i+29],
                     a30_0=A0[i+30], a31_0=A0[i+31];
         register float y0=Y[i+0], y1=Y[i+1], y2=Y[i+2], y3=Y[i+3], y4=Y[i+4],
                        y5=Y[i+5], y6=Y[i+6], y7=Y[i+7], y8=Y[i+8], y9=Y[i+9],
                        y10=Y[i+10], y11=Y[i+11], y12=Y[i+12], y13=Y[i+13],
                        y14=Y[i+14], y15=Y[i+15], y16=Y[i+16], y17=Y[i+17],
                        y18=Y[i+18], y19=Y[i+19], y20=Y[i+20], y21=Y[i+21],
                        y22=Y[i+22], y23=Y[i+23], y24=Y[i+24], y25=Y[i+25],
                        y26=Y[i+26], y27=Y[i+27], y28=Y[i+28], y29=Y[i+29],
                        y30=Y[i+30], y31=Y[i+31];

         prefX(X);
         y0 += a0_0 * x0;
         prefA(A0);
         y1 += a1_0 * x0;
         y2 += a2_0 * x0;
         y3 += a3_0 * x0;
         y4 += a4_0 * x0;
         y5 += a5_0 * x0;
         y6 += a6_0 * x0;
         y7 += a7_0 * x0;
         y8 += a8_0 * x0;
         y9 += a9_0 * x0;
         y10 += a10_0 * x0;
         y11 += a11_0 * x0;
         y12 += a12_0 * x0;
         y13 += a13_0 * x0;
         y14 += a14_0 * x0;
         y15 += a15_0 * x0;
         y16 += a16_0 * x0;
         prefA(A0);
         y17 += a17_0 * x0;
         y18 += a18_0 * x0;
         y19 += a19_0 * x0;
         y20 += a20_0 * x0;
         y21 += a21_0 * x0;
         y22 += a22_0 * x0;
         y23 += a23_0 * x0;
         y24 += a24_0 * x0;
         y25 += a25_0 * x0;
         y26 += a26_0 * x0;
         y27 += a27_0 * x0;
         y28 += a28_0 * x0;
         y29 += a29_0 * x0;
         y30 += a30_0 * x0;
         y31 += a31_0 * x0;
         Y[i+0] = y0;
         Y[i+1] = y1;
         Y[i+2] = y2;
         Y[i+3] = y3;
         Y[i+4] = y4;
         Y[i+5] = y5;
         Y[i+6] = y6;
         Y[i+7] = y7;
         Y[i+8] = y8;
         Y[i+9] = y9;
         Y[i+10] = y10;
         Y[i+11] = y11;
         Y[i+12] = y12;
         Y[i+13] = y13;
         Y[i+14] = y14;
         Y[i+15] = y15;
         Y[i+16] = y16;
         Y[i+17] = y17;
         Y[i+18] = y18;
         Y[i+19] = y19;
         Y[i+20] = y20;
         Y[i+21] = y21;
         Y[i+22] = y22;
         Y[i+23] = y23;
         Y[i+24] = y24;
         Y[i+25] = y25;
         Y[i+26] = y26;
         Y[i+27] = y27;
         Y[i+28] = y28;
         Y[i+29] = y29;
         Y[i+30] = y30;
         Y[i+31] = y31;
      }
      for (i=M32; i < M; i++)
      {
         const float a0_0=A0[i];
         register float y0 = Y[i];

         y0 += a0_0 * x0;
         Y[i] = y0;
      }
   }
}
