#include "atlas_misc.h"
#ifndef PFYDIST
   #define PFYDIST 20
#endif
#ifndef PFXDIST
   #define PFXDIST 0
#endif
#ifndef PFADIST
   #define PFADIST 0
#endif

#if !defined(ATL_3DNow) && !defined(ATL_SSE1) && \
    (PFADIST != 0 || PFXDIST != 0 || PFYDIST != 0)
   #include "atlas_prefetch.h"
#endif

#if defined(ATL_3DNow) || defined(ATL_SSE1)
   #ifndef PFIY
      #define PFIY prefetchnta
   #endif
   #ifndef PFIX
      #define PFIX prefetchnta
   #endif
   #ifndef PFIA
      #ifdef ATL_3DNow
         #define PFIA prefetchw
      #else
         #define PFIA prefetcht0
      #endif
   #endif
#endif
/*
 * X & A are prefetched in M loop PF[A,X]DIST (in bytes) ahead
 */
#if PFADIST == 0                /* flag for no prefetch */
   #define prefA(mem)
#else
   #if defined(ATL_3DNow) || defined(ATL_SSE1)
      #define prefA(mem) __asm__ __volatile__ \
         (Mstr(PFIA) " %0" : : "m" (*(((char *)(mem))+PFADIST)))
   #else
      #if PFLVL == 2
         #define prefA(mem) ATL_pfl2W(((char*)mem)+PFADIST)
      #else
         #define prefA(mem) ATL_pfl1W(((char*)mem)+PFADIST)
      #endif
   #endif
#endif
#if PFXDIST == 0                /* flag for no prefetch */
   #define prefX(mem)
#else
   #if defined(ATL_3DNow) || defined(ATL_SSE1)
      #define prefX(mem) __asm__ __volatile__ \
         (Mstr(PFIX) " %0" : : "m" (*(((char *)(mem))+PFXDIST)))
   #else
      #if PFLVL == 2
         #define prefX(mem) ATL_pfl2R(((char*)mem)+PFXDIST)
      #else
         #define prefX(mem) ATL_pfl1R(((char*)mem)+PFXDIST)
      #endif
   #endif
#endif
/*
 * Y is prefetched in N-loop, and always fetches next NU piece
 */
#if PFYDIST == 0                /* flag for no prefetch */
   #define prefY(mem)
#else
   #if defined(ATL_3DNow) || defined(ATL_SSE1)
      #define prefY(mem) \
         __asm__ __volatile__ (Mstr(PFIY) " %0" : : "m" (*(((char *)(mem)))))
   #else
      #if PFLVL == 2
         #define prefY(mem) ATL_pfl2R(mem)
      #else
         #define prefY(mem) ATL_pfl1R(mem)
      #endif
   #endif
#endif
#ifndef ATL_CINT
   #define ATL_CINT const int
#endif
#ifndef ATL_INT
   #define ATL_INT int
#endif
void ATL_UGER2K(ATL_CINT M, ATL_CINT N, const double *X, const double *Y, 
                const double *W, const double *Z, double *A, ATL_CINT lda)
/*
 * 16x20 unrolled r2_c.
 * Extracted from ATLAS/tune/blas/gemv/atlas-l2g.base
 */
{
   ATL_CINT N20=(N/20)*20, M16=(M/16)*16, lda20=lda*20;
   ATL_INT j;

   for (j=N20; j; j -= 20, A += lda20, Y += 20)
   {
      double *A0=A, *A1=A0+lda, *A2=A1+lda, *A3=A2+lda, *A4=A3+lda, *A5=A4+lda,
             *A6=A5+lda, *A7=A6+lda, *A8=A7+lda, *A9=A8+lda, *A10=A9+lda,
             *A11=A10+lda, *A12=A11+lda, *A13=A12+lda, *A14=A13+lda,
             *A15=A14+lda, *A16=A15+lda, *A17=A16+lda, *A18=A17+lda,
             *A19=A18+lda;
      const register double y0=Y[0], z0=Z[0], y1=Y[1], z1=Z[1], y2=Y[2],
                            z2=Z[2], y3=Y[3], z3=Z[3], y4=Y[4], z4=Z[4],
                            y5=Y[5], z5=Z[5], y6=Y[6], z6=Z[6], y7=Y[7],
                            z7=Z[7], y8=Y[8], z8=Z[8], y9=Y[9], z9=Z[9],
                            y10=Y[10], z10=Z[10], y11=Y[11], z11=Z[11],
                            y12=Y[12], z12=Z[12], y13=Y[13], z13=Z[13],
                            y14=Y[14], z14=Z[14], y15=Y[15], z15=Z[15],
                            y16=Y[16], z16=Z[16], y17=Y[17], z17=Z[17],
                            y18=Y[18], z18=Z[18], y19=Y[19], z19=Z[19];
      ATL_INT i;
      prefY(Y+20+20-1);
      prefY(Z+20+20-1);
      for (i=0; i < M16; i += 16)
      {
         register double a0, a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11, a12,
                         a13, a14, a15, a16, a17, a18, a19;
         const double x0=X[i+0], w0=W[i+0], x1=X[i+1], w1=W[i+1], x2=X[i+2],
                      w2=W[i+2], x3=X[i+3], w3=W[i+3], x4=X[i+4], w4=W[i+4],
                      x5=X[i+5], w5=W[i+5], x6=X[i+6], w6=W[i+6], x7=X[i+7],
                      w7=W[i+7], x8=X[i+8], w8=W[i+8], x9=X[i+9], w9=W[i+9],
                      x10=X[i+10], w10=W[i+10], x11=X[i+11], w11=W[i+11],
                      x12=X[i+12], w12=W[i+12], x13=X[i+13], w13=W[i+13],
                      x14=X[i+14], w14=W[i+14], x15=X[i+15], w15=W[i+15];

         prefX(X);
         a0 = A0[i+0];
         a0 += x0 * y0;
         a1 = A1[i+0];
         a1 += x0 * y1;
         a2 = A2[i+0];
         a2 += x0 * y2;
         a3 = A3[i+0];
         a3 += x0 * y3;
         a4 = A4[i+0];
         a4 += x0 * y4;
         a5 = A5[i+0];
         a5 += x0 * y5;
         a6 = A6[i+0];
         a6 += x0 * y6;
         a7 = A7[i+0];
         a7 += x0 * y7;
         a8 = A8[i+0];
         a8 += x0 * y8;
         a9 = A9[i+0];
         a9 += x0 * y9;
         a10 = A10[i+0];
         a10 += x0 * y10;
         a11 = A11[i+0];
         a11 += x0 * y11;
         a12 = A12[i+0];
         a12 += x0 * y12;
         a13 = A13[i+0];
         a13 += x0 * y13;
         a14 = A14[i+0];
         a14 += x0 * y14;
         a15 = A15[i+0];
         a15 += x0 * y15;
         a16 = A16[i+0];
         a16 += x0 * y16;
         a17 = A17[i+0];
         a17 += x0 * y17;
         a18 = A18[i+0];
         a18 += x0 * y18;
         a19 = A19[i+0];
         a19 += x0 * y19;
         prefX(W);
         a0 += w0 * z0;
         prefA(A0);
         A0[i+0] = a0;
         a1 += w0 * z1;
         prefA(A1);
         A1[i+0] = a1;
         a2 += w0 * z2;
         prefA(A2);
         A2[i+0] = a2;
         a3 += w0 * z3;
         prefA(A3);
         A3[i+0] = a3;
         a4 += w0 * z4;
         prefA(A4);
         A4[i+0] = a4;
         a5 += w0 * z5;
         prefA(A5);
         A5[i+0] = a5;
         a6 += w0 * z6;
         prefA(A6);
         A6[i+0] = a6;
         a7 += w0 * z7;
         prefA(A7);
         A7[i+0] = a7;
         a8 += w0 * z8;
         prefA(A8);
         A8[i+0] = a8;
         a9 += w0 * z9;
         prefA(A9);
         A9[i+0] = a9;
         a10 += w0 * z10;
         prefA(A10);
         A10[i+0] = a10;
         a11 += w0 * z11;
         prefA(A11);
         A11[i+0] = a11;
         a12 += w0 * z12;
         prefA(A12);
         A12[i+0] = a12;
         a13 += w0 * z13;
         prefA(A13);
         A13[i+0] = a13;
         a14 += w0 * z14;
         prefA(A14);
         A14[i+0] = a14;
         a15 += w0 * z15;
         prefA(A15);
         A15[i+0] = a15;
         a16 += w0 * z16;
         prefA(A16);
         A16[i+0] = a16;
         a17 += w0 * z17;
         prefA(A17);
         A17[i+0] = a17;
         a18 += w0 * z18;
         prefA(A18);
         A18[i+0] = a18;
         a19 += w0 * z19;
         prefA(A19);
         A19[i+0] = a19;
         a0 = A0[i+1];
         a0 += x1 * y0;
         a1 = A1[i+1];
         a1 += x1 * y1;
         a2 = A2[i+1];
         a2 += x1 * y2;
         a3 = A3[i+1];
         a3 += x1 * y3;
         a4 = A4[i+1];
         a4 += x1 * y4;
         a5 = A5[i+1];
         a5 += x1 * y5;
         a6 = A6[i+1];
         a6 += x1 * y6;
         a7 = A7[i+1];
         a7 += x1 * y7;
         a8 = A8[i+1];
         a8 += x1 * y8;
         a9 = A9[i+1];
         a9 += x1 * y9;
         a10 = A10[i+1];
         a10 += x1 * y10;
         a11 = A11[i+1];
         a11 += x1 * y11;
         a12 = A12[i+1];
         a12 += x1 * y12;
         a13 = A13[i+1];
         a13 += x1 * y13;
         a14 = A14[i+1];
         a14 += x1 * y14;
         a15 = A15[i+1];
         a15 += x1 * y15;
         a16 = A16[i+1];
         a16 += x1 * y16;
         a17 = A17[i+1];
         a17 += x1 * y17;
         a18 = A18[i+1];
         a18 += x1 * y18;
         a19 = A19[i+1];
         a19 += x1 * y19;
         a0 += w1 * z0;
         A0[i+1] = a0;
         a1 += w1 * z1;
         A1[i+1] = a1;
         a2 += w1 * z2;
         A2[i+1] = a2;
         a3 += w1 * z3;
         A3[i+1] = a3;
         a4 += w1 * z4;
         A4[i+1] = a4;
         a5 += w1 * z5;
         A5[i+1] = a5;
         a6 += w1 * z6;
         A6[i+1] = a6;
         a7 += w1 * z7;
         A7[i+1] = a7;
         a8 += w1 * z8;
         A8[i+1] = a8;
         a9 += w1 * z9;
         A9[i+1] = a9;
         a10 += w1 * z10;
         A10[i+1] = a10;
         a11 += w1 * z11;
         A11[i+1] = a11;
         a12 += w1 * z12;
         A12[i+1] = a12;
         a13 += w1 * z13;
         A13[i+1] = a13;
         a14 += w1 * z14;
         A14[i+1] = a14;
         a15 += w1 * z15;
         A15[i+1] = a15;
         a16 += w1 * z16;
         A16[i+1] = a16;
         a17 += w1 * z17;
         A17[i+1] = a17;
         a18 += w1 * z18;
         A18[i+1] = a18;
         a19 += w1 * z19;
         A19[i+1] = a19;
         a0 = A0[i+2];
         a0 += x2 * y0;
         a1 = A1[i+2];
         a1 += x2 * y1;
         a2 = A2[i+2];
         a2 += x2 * y2;
         a3 = A3[i+2];
         a3 += x2 * y3;
         a4 = A4[i+2];
         a4 += x2 * y4;
         a5 = A5[i+2];
         a5 += x2 * y5;
         a6 = A6[i+2];
         a6 += x2 * y6;
         a7 = A7[i+2];
         a7 += x2 * y7;
         a8 = A8[i+2];
         a8 += x2 * y8;
         a9 = A9[i+2];
         a9 += x2 * y9;
         a10 = A10[i+2];
         a10 += x2 * y10;
         a11 = A11[i+2];
         a11 += x2 * y11;
         a12 = A12[i+2];
         a12 += x2 * y12;
         a13 = A13[i+2];
         a13 += x2 * y13;
         a14 = A14[i+2];
         a14 += x2 * y14;
         a15 = A15[i+2];
         a15 += x2 * y15;
         a16 = A16[i+2];
         a16 += x2 * y16;
         a17 = A17[i+2];
         a17 += x2 * y17;
         a18 = A18[i+2];
         a18 += x2 * y18;
         a19 = A19[i+2];
         a19 += x2 * y19;
         a0 += w2 * z0;
         A0[i+2] = a0;
         a1 += w2 * z1;
         A1[i+2] = a1;
         a2 += w2 * z2;
         A2[i+2] = a2;
         a3 += w2 * z3;
         A3[i+2] = a3;
         a4 += w2 * z4;
         A4[i+2] = a4;
         a5 += w2 * z5;
         A5[i+2] = a5;
         a6 += w2 * z6;
         A6[i+2] = a6;
         a7 += w2 * z7;
         A7[i+2] = a7;
         a8 += w2 * z8;
         A8[i+2] = a8;
         a9 += w2 * z9;
         A9[i+2] = a9;
         a10 += w2 * z10;
         A10[i+2] = a10;
         a11 += w2 * z11;
         A11[i+2] = a11;
         a12 += w2 * z12;
         A12[i+2] = a12;
         a13 += w2 * z13;
         A13[i+2] = a13;
         a14 += w2 * z14;
         A14[i+2] = a14;
         a15 += w2 * z15;
         A15[i+2] = a15;
         a16 += w2 * z16;
         A16[i+2] = a16;
         a17 += w2 * z17;
         A17[i+2] = a17;
         a18 += w2 * z18;
         A18[i+2] = a18;
         a19 += w2 * z19;
         A19[i+2] = a19;
         a0 = A0[i+3];
         a0 += x3 * y0;
         a1 = A1[i+3];
         a1 += x3 * y1;
         a2 = A2[i+3];
         a2 += x3 * y2;
         a3 = A3[i+3];
         a3 += x3 * y3;
         a4 = A4[i+3];
         a4 += x3 * y4;
         a5 = A5[i+3];
         a5 += x3 * y5;
         a6 = A6[i+3];
         a6 += x3 * y6;
         a7 = A7[i+3];
         a7 += x3 * y7;
         a8 = A8[i+3];
         a8 += x3 * y8;
         a9 = A9[i+3];
         a9 += x3 * y9;
         a10 = A10[i+3];
         a10 += x3 * y10;
         a11 = A11[i+3];
         a11 += x3 * y11;
         a12 = A12[i+3];
         a12 += x3 * y12;
         a13 = A13[i+3];
         a13 += x3 * y13;
         a14 = A14[i+3];
         a14 += x3 * y14;
         a15 = A15[i+3];
         a15 += x3 * y15;
         a16 = A16[i+3];
         a16 += x3 * y16;
         a17 = A17[i+3];
         a17 += x3 * y17;
         a18 = A18[i+3];
         a18 += x3 * y18;
         a19 = A19[i+3];
         a19 += x3 * y19;
         a0 += w3 * z0;
         A0[i+3] = a0;
         a1 += w3 * z1;
         A1[i+3] = a1;
         a2 += w3 * z2;
         A2[i+3] = a2;
         a3 += w3 * z3;
         A3[i+3] = a3;
         a4 += w3 * z4;
         A4[i+3] = a4;
         a5 += w3 * z5;
         A5[i+3] = a5;
         a6 += w3 * z6;
         A6[i+3] = a6;
         a7 += w3 * z7;
         A7[i+3] = a7;
         a8 += w3 * z8;
         A8[i+3] = a8;
         a9 += w3 * z9;
         A9[i+3] = a9;
         a10 += w3 * z10;
         A10[i+3] = a10;
         a11 += w3 * z11;
         A11[i+3] = a11;
         a12 += w3 * z12;
         A12[i+3] = a12;
         a13 += w3 * z13;
         A13[i+3] = a13;
         a14 += w3 * z14;
         A14[i+3] = a14;
         a15 += w3 * z15;
         A15[i+3] = a15;
         a16 += w3 * z16;
         A16[i+3] = a16;
         a17 += w3 * z17;
         A17[i+3] = a17;
         a18 += w3 * z18;
         A18[i+3] = a18;
         a19 += w3 * z19;
         A19[i+3] = a19;
         a0 = A0[i+4];
         a0 += x4 * y0;
         a1 = A1[i+4];
         a1 += x4 * y1;
         a2 = A2[i+4];
         a2 += x4 * y2;
         a3 = A3[i+4];
         a3 += x4 * y3;
         a4 = A4[i+4];
         a4 += x4 * y4;
         a5 = A5[i+4];
         a5 += x4 * y5;
         a6 = A6[i+4];
         a6 += x4 * y6;
         a7 = A7[i+4];
         a7 += x4 * y7;
         a8 = A8[i+4];
         a8 += x4 * y8;
         a9 = A9[i+4];
         a9 += x4 * y9;
         a10 = A10[i+4];
         a10 += x4 * y10;
         a11 = A11[i+4];
         a11 += x4 * y11;
         a12 = A12[i+4];
         a12 += x4 * y12;
         a13 = A13[i+4];
         a13 += x4 * y13;
         a14 = A14[i+4];
         a14 += x4 * y14;
         a15 = A15[i+4];
         a15 += x4 * y15;
         a16 = A16[i+4];
         a16 += x4 * y16;
         a17 = A17[i+4];
         a17 += x4 * y17;
         a18 = A18[i+4];
         a18 += x4 * y18;
         a19 = A19[i+4];
         a19 += x4 * y19;
         a0 += w4 * z0;
         A0[i+4] = a0;
         a1 += w4 * z1;
         A1[i+4] = a1;
         a2 += w4 * z2;
         A2[i+4] = a2;
         a3 += w4 * z3;
         A3[i+4] = a3;
         a4 += w4 * z4;
         A4[i+4] = a4;
         a5 += w4 * z5;
         A5[i+4] = a5;
         a6 += w4 * z6;
         A6[i+4] = a6;
         a7 += w4 * z7;
         A7[i+4] = a7;
         a8 += w4 * z8;
         A8[i+4] = a8;
         a9 += w4 * z9;
         A9[i+4] = a9;
         a10 += w4 * z10;
         A10[i+4] = a10;
         a11 += w4 * z11;
         A11[i+4] = a11;
         a12 += w4 * z12;
         A12[i+4] = a12;
         a13 += w4 * z13;
         A13[i+4] = a13;
         a14 += w4 * z14;
         A14[i+4] = a14;
         a15 += w4 * z15;
         A15[i+4] = a15;
         a16 += w4 * z16;
         A16[i+4] = a16;
         a17 += w4 * z17;
         A17[i+4] = a17;
         a18 += w4 * z18;
         A18[i+4] = a18;
         a19 += w4 * z19;
         A19[i+4] = a19;
         a0 = A0[i+5];
         a0 += x5 * y0;
         a1 = A1[i+5];
         a1 += x5 * y1;
         a2 = A2[i+5];
         a2 += x5 * y2;
         a3 = A3[i+5];
         a3 += x5 * y3;
         a4 = A4[i+5];
         a4 += x5 * y4;
         a5 = A5[i+5];
         a5 += x5 * y5;
         a6 = A6[i+5];
         a6 += x5 * y6;
         a7 = A7[i+5];
         a7 += x5 * y7;
         a8 = A8[i+5];
         a8 += x5 * y8;
         a9 = A9[i+5];
         a9 += x5 * y9;
         a10 = A10[i+5];
         a10 += x5 * y10;
         a11 = A11[i+5];
         a11 += x5 * y11;
         a12 = A12[i+5];
         a12 += x5 * y12;
         a13 = A13[i+5];
         a13 += x5 * y13;
         a14 = A14[i+5];
         a14 += x5 * y14;
         a15 = A15[i+5];
         a15 += x5 * y15;
         a16 = A16[i+5];
         a16 += x5 * y16;
         a17 = A17[i+5];
         a17 += x5 * y17;
         a18 = A18[i+5];
         a18 += x5 * y18;
         a19 = A19[i+5];
         a19 += x5 * y19;
         a0 += w5 * z0;
         A0[i+5] = a0;
         a1 += w5 * z1;
         A1[i+5] = a1;
         a2 += w5 * z2;
         A2[i+5] = a2;
         a3 += w5 * z3;
         A3[i+5] = a3;
         a4 += w5 * z4;
         A4[i+5] = a4;
         a5 += w5 * z5;
         A5[i+5] = a5;
         a6 += w5 * z6;
         A6[i+5] = a6;
         a7 += w5 * z7;
         A7[i+5] = a7;
         a8 += w5 * z8;
         A8[i+5] = a8;
         a9 += w5 * z9;
         A9[i+5] = a9;
         a10 += w5 * z10;
         A10[i+5] = a10;
         a11 += w5 * z11;
         A11[i+5] = a11;
         a12 += w5 * z12;
         A12[i+5] = a12;
         a13 += w5 * z13;
         A13[i+5] = a13;
         a14 += w5 * z14;
         A14[i+5] = a14;
         a15 += w5 * z15;
         A15[i+5] = a15;
         a16 += w5 * z16;
         A16[i+5] = a16;
         a17 += w5 * z17;
         A17[i+5] = a17;
         a18 += w5 * z18;
         A18[i+5] = a18;
         a19 += w5 * z19;
         A19[i+5] = a19;
         a0 = A0[i+6];
         a0 += x6 * y0;
         a1 = A1[i+6];
         a1 += x6 * y1;
         a2 = A2[i+6];
         a2 += x6 * y2;
         a3 = A3[i+6];
         a3 += x6 * y3;
         a4 = A4[i+6];
         a4 += x6 * y4;
         a5 = A5[i+6];
         a5 += x6 * y5;
         a6 = A6[i+6];
         a6 += x6 * y6;
         a7 = A7[i+6];
         a7 += x6 * y7;
         a8 = A8[i+6];
         a8 += x6 * y8;
         a9 = A9[i+6];
         a9 += x6 * y9;
         a10 = A10[i+6];
         a10 += x6 * y10;
         a11 = A11[i+6];
         a11 += x6 * y11;
         a12 = A12[i+6];
         a12 += x6 * y12;
         a13 = A13[i+6];
         a13 += x6 * y13;
         a14 = A14[i+6];
         a14 += x6 * y14;
         a15 = A15[i+6];
         a15 += x6 * y15;
         a16 = A16[i+6];
         a16 += x6 * y16;
         a17 = A17[i+6];
         a17 += x6 * y17;
         a18 = A18[i+6];
         a18 += x6 * y18;
         a19 = A19[i+6];
         a19 += x6 * y19;
         a0 += w6 * z0;
         A0[i+6] = a0;
         a1 += w6 * z1;
         A1[i+6] = a1;
         a2 += w6 * z2;
         A2[i+6] = a2;
         a3 += w6 * z3;
         A3[i+6] = a3;
         a4 += w6 * z4;
         A4[i+6] = a4;
         a5 += w6 * z5;
         A5[i+6] = a5;
         a6 += w6 * z6;
         A6[i+6] = a6;
         a7 += w6 * z7;
         A7[i+6] = a7;
         a8 += w6 * z8;
         A8[i+6] = a8;
         a9 += w6 * z9;
         A9[i+6] = a9;
         a10 += w6 * z10;
         A10[i+6] = a10;
         a11 += w6 * z11;
         A11[i+6] = a11;
         a12 += w6 * z12;
         A12[i+6] = a12;
         a13 += w6 * z13;
         A13[i+6] = a13;
         a14 += w6 * z14;
         A14[i+6] = a14;
         a15 += w6 * z15;
         A15[i+6] = a15;
         a16 += w6 * z16;
         A16[i+6] = a16;
         a17 += w6 * z17;
         A17[i+6] = a17;
         a18 += w6 * z18;
         A18[i+6] = a18;
         a19 += w6 * z19;
         A19[i+6] = a19;
         a0 = A0[i+7];
         a0 += x7 * y0;
         a1 = A1[i+7];
         a1 += x7 * y1;
         a2 = A2[i+7];
         a2 += x7 * y2;
         a3 = A3[i+7];
         a3 += x7 * y3;
         a4 = A4[i+7];
         a4 += x7 * y4;
         a5 = A5[i+7];
         a5 += x7 * y5;
         a6 = A6[i+7];
         a6 += x7 * y6;
         a7 = A7[i+7];
         a7 += x7 * y7;
         a8 = A8[i+7];
         a8 += x7 * y8;
         a9 = A9[i+7];
         a9 += x7 * y9;
         a10 = A10[i+7];
         a10 += x7 * y10;
         a11 = A11[i+7];
         a11 += x7 * y11;
         a12 = A12[i+7];
         a12 += x7 * y12;
         a13 = A13[i+7];
         a13 += x7 * y13;
         a14 = A14[i+7];
         a14 += x7 * y14;
         a15 = A15[i+7];
         a15 += x7 * y15;
         a16 = A16[i+7];
         a16 += x7 * y16;
         a17 = A17[i+7];
         a17 += x7 * y17;
         a18 = A18[i+7];
         a18 += x7 * y18;
         a19 = A19[i+7];
         a19 += x7 * y19;
         a0 += w7 * z0;
         A0[i+7] = a0;
         a1 += w7 * z1;
         A1[i+7] = a1;
         a2 += w7 * z2;
         A2[i+7] = a2;
         a3 += w7 * z3;
         A3[i+7] = a3;
         a4 += w7 * z4;
         A4[i+7] = a4;
         a5 += w7 * z5;
         A5[i+7] = a5;
         a6 += w7 * z6;
         A6[i+7] = a6;
         a7 += w7 * z7;
         A7[i+7] = a7;
         a8 += w7 * z8;
         A8[i+7] = a8;
         a9 += w7 * z9;
         A9[i+7] = a9;
         a10 += w7 * z10;
         A10[i+7] = a10;
         a11 += w7 * z11;
         A11[i+7] = a11;
         a12 += w7 * z12;
         A12[i+7] = a12;
         a13 += w7 * z13;
         A13[i+7] = a13;
         a14 += w7 * z14;
         A14[i+7] = a14;
         a15 += w7 * z15;
         A15[i+7] = a15;
         a16 += w7 * z16;
         A16[i+7] = a16;
         a17 += w7 * z17;
         A17[i+7] = a17;
         a18 += w7 * z18;
         A18[i+7] = a18;
         a19 += w7 * z19;
         A19[i+7] = a19;
         a0 = A0[i+8];
         a0 += x8 * y0;
         a1 = A1[i+8];
         a1 += x8 * y1;
         a2 = A2[i+8];
         a2 += x8 * y2;
         a3 = A3[i+8];
         a3 += x8 * y3;
         a4 = A4[i+8];
         a4 += x8 * y4;
         a5 = A5[i+8];
         a5 += x8 * y5;
         a6 = A6[i+8];
         a6 += x8 * y6;
         a7 = A7[i+8];
         a7 += x8 * y7;
         a8 = A8[i+8];
         a8 += x8 * y8;
         a9 = A9[i+8];
         a9 += x8 * y9;
         a10 = A10[i+8];
         a10 += x8 * y10;
         a11 = A11[i+8];
         a11 += x8 * y11;
         a12 = A12[i+8];
         a12 += x8 * y12;
         a13 = A13[i+8];
         a13 += x8 * y13;
         a14 = A14[i+8];
         a14 += x8 * y14;
         a15 = A15[i+8];
         a15 += x8 * y15;
         a16 = A16[i+8];
         a16 += x8 * y16;
         a17 = A17[i+8];
         a17 += x8 * y17;
         a18 = A18[i+8];
         a18 += x8 * y18;
         a19 = A19[i+8];
         a19 += x8 * y19;
         a0 += w8 * z0;
         prefA(A0);
         A0[i+8] = a0;
         a1 += w8 * z1;
         prefA(A1);
         A1[i+8] = a1;
         a2 += w8 * z2;
         prefA(A2);
         A2[i+8] = a2;
         a3 += w8 * z3;
         prefA(A3);
         A3[i+8] = a3;
         a4 += w8 * z4;
         prefA(A4);
         A4[i+8] = a4;
         a5 += w8 * z5;
         prefA(A5);
         A5[i+8] = a5;
         a6 += w8 * z6;
         prefA(A6);
         A6[i+8] = a6;
         a7 += w8 * z7;
         prefA(A7);
         A7[i+8] = a7;
         a8 += w8 * z8;
         prefA(A8);
         A8[i+8] = a8;
         a9 += w8 * z9;
         prefA(A9);
         A9[i+8] = a9;
         a10 += w8 * z10;
         prefA(A10);
         A10[i+8] = a10;
         a11 += w8 * z11;
         prefA(A11);
         A11[i+8] = a11;
         a12 += w8 * z12;
         prefA(A12);
         A12[i+8] = a12;
         a13 += w8 * z13;
         prefA(A13);
         A13[i+8] = a13;
         a14 += w8 * z14;
         prefA(A14);
         A14[i+8] = a14;
         a15 += w8 * z15;
         prefA(A15);
         A15[i+8] = a15;
         a16 += w8 * z16;
         prefA(A16);
         A16[i+8] = a16;
         a17 += w8 * z17;
         prefA(A17);
         A17[i+8] = a17;
         a18 += w8 * z18;
         prefA(A18);
         A18[i+8] = a18;
         a19 += w8 * z19;
         prefA(A19);
         A19[i+8] = a19;
         a0 = A0[i+9];
         a0 += x9 * y0;
         a1 = A1[i+9];
         a1 += x9 * y1;
         a2 = A2[i+9];
         a2 += x9 * y2;
         a3 = A3[i+9];
         a3 += x9 * y3;
         a4 = A4[i+9];
         a4 += x9 * y4;
         a5 = A5[i+9];
         a5 += x9 * y5;
         a6 = A6[i+9];
         a6 += x9 * y6;
         a7 = A7[i+9];
         a7 += x9 * y7;
         a8 = A8[i+9];
         a8 += x9 * y8;
         a9 = A9[i+9];
         a9 += x9 * y9;
         a10 = A10[i+9];
         a10 += x9 * y10;
         a11 = A11[i+9];
         a11 += x9 * y11;
         a12 = A12[i+9];
         a12 += x9 * y12;
         a13 = A13[i+9];
         a13 += x9 * y13;
         a14 = A14[i+9];
         a14 += x9 * y14;
         a15 = A15[i+9];
         a15 += x9 * y15;
         a16 = A16[i+9];
         a16 += x9 * y16;
         a17 = A17[i+9];
         a17 += x9 * y17;
         a18 = A18[i+9];
         a18 += x9 * y18;
         a19 = A19[i+9];
         a19 += x9 * y19;
         a0 += w9 * z0;
         A0[i+9] = a0;
         a1 += w9 * z1;
         A1[i+9] = a1;
         a2 += w9 * z2;
         A2[i+9] = a2;
         a3 += w9 * z3;
         A3[i+9] = a3;
         a4 += w9 * z4;
         A4[i+9] = a4;
         a5 += w9 * z5;
         A5[i+9] = a5;
         a6 += w9 * z6;
         A6[i+9] = a6;
         a7 += w9 * z7;
         A7[i+9] = a7;
         a8 += w9 * z8;
         A8[i+9] = a8;
         a9 += w9 * z9;
         A9[i+9] = a9;
         a10 += w9 * z10;
         A10[i+9] = a10;
         a11 += w9 * z11;
         A11[i+9] = a11;
         a12 += w9 * z12;
         A12[i+9] = a12;
         a13 += w9 * z13;
         A13[i+9] = a13;
         a14 += w9 * z14;
         A14[i+9] = a14;
         a15 += w9 * z15;
         A15[i+9] = a15;
         a16 += w9 * z16;
         A16[i+9] = a16;
         a17 += w9 * z17;
         A17[i+9] = a17;
         a18 += w9 * z18;
         A18[i+9] = a18;
         a19 += w9 * z19;
         A19[i+9] = a19;
         a0 = A0[i+10];
         a0 += x10 * y0;
         a1 = A1[i+10];
         a1 += x10 * y1;
         a2 = A2[i+10];
         a2 += x10 * y2;
         a3 = A3[i+10];
         a3 += x10 * y3;
         a4 = A4[i+10];
         a4 += x10 * y4;
         a5 = A5[i+10];
         a5 += x10 * y5;
         a6 = A6[i+10];
         a6 += x10 * y6;
         a7 = A7[i+10];
         a7 += x10 * y7;
         a8 = A8[i+10];
         a8 += x10 * y8;
         a9 = A9[i+10];
         a9 += x10 * y9;
         a10 = A10[i+10];
         a10 += x10 * y10;
         a11 = A11[i+10];
         a11 += x10 * y11;
         a12 = A12[i+10];
         a12 += x10 * y12;
         a13 = A13[i+10];
         a13 += x10 * y13;
         a14 = A14[i+10];
         a14 += x10 * y14;
         a15 = A15[i+10];
         a15 += x10 * y15;
         a16 = A16[i+10];
         a16 += x10 * y16;
         a17 = A17[i+10];
         a17 += x10 * y17;
         a18 = A18[i+10];
         a18 += x10 * y18;
         a19 = A19[i+10];
         a19 += x10 * y19;
         a0 += w10 * z0;
         A0[i+10] = a0;
         a1 += w10 * z1;
         A1[i+10] = a1;
         a2 += w10 * z2;
         A2[i+10] = a2;
         a3 += w10 * z3;
         A3[i+10] = a3;
         a4 += w10 * z4;
         A4[i+10] = a4;
         a5 += w10 * z5;
         A5[i+10] = a5;
         a6 += w10 * z6;
         A6[i+10] = a6;
         a7 += w10 * z7;
         A7[i+10] = a7;
         a8 += w10 * z8;
         A8[i+10] = a8;
         a9 += w10 * z9;
         A9[i+10] = a9;
         a10 += w10 * z10;
         A10[i+10] = a10;
         a11 += w10 * z11;
         A11[i+10] = a11;
         a12 += w10 * z12;
         A12[i+10] = a12;
         a13 += w10 * z13;
         A13[i+10] = a13;
         a14 += w10 * z14;
         A14[i+10] = a14;
         a15 += w10 * z15;
         A15[i+10] = a15;
         a16 += w10 * z16;
         A16[i+10] = a16;
         a17 += w10 * z17;
         A17[i+10] = a17;
         a18 += w10 * z18;
         A18[i+10] = a18;
         a19 += w10 * z19;
         A19[i+10] = a19;
         a0 = A0[i+11];
         a0 += x11 * y0;
         a1 = A1[i+11];
         a1 += x11 * y1;
         a2 = A2[i+11];
         a2 += x11 * y2;
         a3 = A3[i+11];
         a3 += x11 * y3;
         a4 = A4[i+11];
         a4 += x11 * y4;
         a5 = A5[i+11];
         a5 += x11 * y5;
         a6 = A6[i+11];
         a6 += x11 * y6;
         a7 = A7[i+11];
         a7 += x11 * y7;
         a8 = A8[i+11];
         a8 += x11 * y8;
         a9 = A9[i+11];
         a9 += x11 * y9;
         a10 = A10[i+11];
         a10 += x11 * y10;
         a11 = A11[i+11];
         a11 += x11 * y11;
         a12 = A12[i+11];
         a12 += x11 * y12;
         a13 = A13[i+11];
         a13 += x11 * y13;
         a14 = A14[i+11];
         a14 += x11 * y14;
         a15 = A15[i+11];
         a15 += x11 * y15;
         a16 = A16[i+11];
         a16 += x11 * y16;
         a17 = A17[i+11];
         a17 += x11 * y17;
         a18 = A18[i+11];
         a18 += x11 * y18;
         a19 = A19[i+11];
         a19 += x11 * y19;
         a0 += w11 * z0;
         A0[i+11] = a0;
         a1 += w11 * z1;
         A1[i+11] = a1;
         a2 += w11 * z2;
         A2[i+11] = a2;
         a3 += w11 * z3;
         A3[i+11] = a3;
         a4 += w11 * z4;
         A4[i+11] = a4;
         a5 += w11 * z5;
         A5[i+11] = a5;
         a6 += w11 * z6;
         A6[i+11] = a6;
         a7 += w11 * z7;
         A7[i+11] = a7;
         a8 += w11 * z8;
         A8[i+11] = a8;
         a9 += w11 * z9;
         A9[i+11] = a9;
         a10 += w11 * z10;
         A10[i+11] = a10;
         a11 += w11 * z11;
         A11[i+11] = a11;
         a12 += w11 * z12;
         A12[i+11] = a12;
         a13 += w11 * z13;
         A13[i+11] = a13;
         a14 += w11 * z14;
         A14[i+11] = a14;
         a15 += w11 * z15;
         A15[i+11] = a15;
         a16 += w11 * z16;
         A16[i+11] = a16;
         a17 += w11 * z17;
         A17[i+11] = a17;
         a18 += w11 * z18;
         A18[i+11] = a18;
         a19 += w11 * z19;
         A19[i+11] = a19;
         a0 = A0[i+12];
         a0 += x12 * y0;
         a1 = A1[i+12];
         a1 += x12 * y1;
         a2 = A2[i+12];
         a2 += x12 * y2;
         a3 = A3[i+12];
         a3 += x12 * y3;
         a4 = A4[i+12];
         a4 += x12 * y4;
         a5 = A5[i+12];
         a5 += x12 * y5;
         a6 = A6[i+12];
         a6 += x12 * y6;
         a7 = A7[i+12];
         a7 += x12 * y7;
         a8 = A8[i+12];
         a8 += x12 * y8;
         a9 = A9[i+12];
         a9 += x12 * y9;
         a10 = A10[i+12];
         a10 += x12 * y10;
         a11 = A11[i+12];
         a11 += x12 * y11;
         a12 = A12[i+12];
         a12 += x12 * y12;
         a13 = A13[i+12];
         a13 += x12 * y13;
         a14 = A14[i+12];
         a14 += x12 * y14;
         a15 = A15[i+12];
         a15 += x12 * y15;
         a16 = A16[i+12];
         a16 += x12 * y16;
         a17 = A17[i+12];
         a17 += x12 * y17;
         a18 = A18[i+12];
         a18 += x12 * y18;
         a19 = A19[i+12];
         a19 += x12 * y19;
         a0 += w12 * z0;
         A0[i+12] = a0;
         a1 += w12 * z1;
         A1[i+12] = a1;
         a2 += w12 * z2;
         A2[i+12] = a2;
         a3 += w12 * z3;
         A3[i+12] = a3;
         a4 += w12 * z4;
         A4[i+12] = a4;
         a5 += w12 * z5;
         A5[i+12] = a5;
         a6 += w12 * z6;
         A6[i+12] = a6;
         a7 += w12 * z7;
         A7[i+12] = a7;
         a8 += w12 * z8;
         A8[i+12] = a8;
         a9 += w12 * z9;
         A9[i+12] = a9;
         a10 += w12 * z10;
         A10[i+12] = a10;
         a11 += w12 * z11;
         A11[i+12] = a11;
         a12 += w12 * z12;
         A12[i+12] = a12;
         a13 += w12 * z13;
         A13[i+12] = a13;
         a14 += w12 * z14;
         A14[i+12] = a14;
         a15 += w12 * z15;
         A15[i+12] = a15;
         a16 += w12 * z16;
         A16[i+12] = a16;
         a17 += w12 * z17;
         A17[i+12] = a17;
         a18 += w12 * z18;
         A18[i+12] = a18;
         a19 += w12 * z19;
         A19[i+12] = a19;
         a0 = A0[i+13];
         a0 += x13 * y0;
         a1 = A1[i+13];
         a1 += x13 * y1;
         a2 = A2[i+13];
         a2 += x13 * y2;
         a3 = A3[i+13];
         a3 += x13 * y3;
         a4 = A4[i+13];
         a4 += x13 * y4;
         a5 = A5[i+13];
         a5 += x13 * y5;
         a6 = A6[i+13];
         a6 += x13 * y6;
         a7 = A7[i+13];
         a7 += x13 * y7;
         a8 = A8[i+13];
         a8 += x13 * y8;
         a9 = A9[i+13];
         a9 += x13 * y9;
         a10 = A10[i+13];
         a10 += x13 * y10;
         a11 = A11[i+13];
         a11 += x13 * y11;
         a12 = A12[i+13];
         a12 += x13 * y12;
         a13 = A13[i+13];
         a13 += x13 * y13;
         a14 = A14[i+13];
         a14 += x13 * y14;
         a15 = A15[i+13];
         a15 += x13 * y15;
         a16 = A16[i+13];
         a16 += x13 * y16;
         a17 = A17[i+13];
         a17 += x13 * y17;
         a18 = A18[i+13];
         a18 += x13 * y18;
         a19 = A19[i+13];
         a19 += x13 * y19;
         a0 += w13 * z0;
         A0[i+13] = a0;
         a1 += w13 * z1;
         A1[i+13] = a1;
         a2 += w13 * z2;
         A2[i+13] = a2;
         a3 += w13 * z3;
         A3[i+13] = a3;
         a4 += w13 * z4;
         A4[i+13] = a4;
         a5 += w13 * z5;
         A5[i+13] = a5;
         a6 += w13 * z6;
         A6[i+13] = a6;
         a7 += w13 * z7;
         A7[i+13] = a7;
         a8 += w13 * z8;
         A8[i+13] = a8;
         a9 += w13 * z9;
         A9[i+13] = a9;
         a10 += w13 * z10;
         A10[i+13] = a10;
         a11 += w13 * z11;
         A11[i+13] = a11;
         a12 += w13 * z12;
         A12[i+13] = a12;
         a13 += w13 * z13;
         A13[i+13] = a13;
         a14 += w13 * z14;
         A14[i+13] = a14;
         a15 += w13 * z15;
         A15[i+13] = a15;
         a16 += w13 * z16;
         A16[i+13] = a16;
         a17 += w13 * z17;
         A17[i+13] = a17;
         a18 += w13 * z18;
         A18[i+13] = a18;
         a19 += w13 * z19;
         A19[i+13] = a19;
         a0 = A0[i+14];
         a0 += x14 * y0;
         a1 = A1[i+14];
         a1 += x14 * y1;
         a2 = A2[i+14];
         a2 += x14 * y2;
         a3 = A3[i+14];
         a3 += x14 * y3;
         a4 = A4[i+14];
         a4 += x14 * y4;
         a5 = A5[i+14];
         a5 += x14 * y5;
         a6 = A6[i+14];
         a6 += x14 * y6;
         a7 = A7[i+14];
         a7 += x14 * y7;
         a8 = A8[i+14];
         a8 += x14 * y8;
         a9 = A9[i+14];
         a9 += x14 * y9;
         a10 = A10[i+14];
         a10 += x14 * y10;
         a11 = A11[i+14];
         a11 += x14 * y11;
         a12 = A12[i+14];
         a12 += x14 * y12;
         a13 = A13[i+14];
         a13 += x14 * y13;
         a14 = A14[i+14];
         a14 += x14 * y14;
         a15 = A15[i+14];
         a15 += x14 * y15;
         a16 = A16[i+14];
         a16 += x14 * y16;
         a17 = A17[i+14];
         a17 += x14 * y17;
         a18 = A18[i+14];
         a18 += x14 * y18;
         a19 = A19[i+14];
         a19 += x14 * y19;
         a0 += w14 * z0;
         A0[i+14] = a0;
         a1 += w14 * z1;
         A1[i+14] = a1;
         a2 += w14 * z2;
         A2[i+14] = a2;
         a3 += w14 * z3;
         A3[i+14] = a3;
         a4 += w14 * z4;
         A4[i+14] = a4;
         a5 += w14 * z5;
         A5[i+14] = a5;
         a6 += w14 * z6;
         A6[i+14] = a6;
         a7 += w14 * z7;
         A7[i+14] = a7;
         a8 += w14 * z8;
         A8[i+14] = a8;
         a9 += w14 * z9;
         A9[i+14] = a9;
         a10 += w14 * z10;
         A10[i+14] = a10;
         a11 += w14 * z11;
         A11[i+14] = a11;
         a12 += w14 * z12;
         A12[i+14] = a12;
         a13 += w14 * z13;
         A13[i+14] = a13;
         a14 += w14 * z14;
         A14[i+14] = a14;
         a15 += w14 * z15;
         A15[i+14] = a15;
         a16 += w14 * z16;
         A16[i+14] = a16;
         a17 += w14 * z17;
         A17[i+14] = a17;
         a18 += w14 * z18;
         A18[i+14] = a18;
         a19 += w14 * z19;
         A19[i+14] = a19;
         a0 = A0[i+15];
         a0 += x15 * y0;
         a1 = A1[i+15];
         a1 += x15 * y1;
         a2 = A2[i+15];
         a2 += x15 * y2;
         a3 = A3[i+15];
         a3 += x15 * y3;
         a4 = A4[i+15];
         a4 += x15 * y4;
         a5 = A5[i+15];
         a5 += x15 * y5;
         a6 = A6[i+15];
         a6 += x15 * y6;
         a7 = A7[i+15];
         a7 += x15 * y7;
         a8 = A8[i+15];
         a8 += x15 * y8;
         a9 = A9[i+15];
         a9 += x15 * y9;
         a10 = A10[i+15];
         a10 += x15 * y10;
         a11 = A11[i+15];
         a11 += x15 * y11;
         a12 = A12[i+15];
         a12 += x15 * y12;
         a13 = A13[i+15];
         a13 += x15 * y13;
         a14 = A14[i+15];
         a14 += x15 * y14;
         a15 = A15[i+15];
         a15 += x15 * y15;
         a16 = A16[i+15];
         a16 += x15 * y16;
         a17 = A17[i+15];
         a17 += x15 * y17;
         a18 = A18[i+15];
         a18 += x15 * y18;
         a19 = A19[i+15];
         a19 += x15 * y19;
         a0 += w15 * z0;
         A0[i+15] = a0;
         a1 += w15 * z1;
         A1[i+15] = a1;
         a2 += w15 * z2;
         A2[i+15] = a2;
         a3 += w15 * z3;
         A3[i+15] = a3;
         a4 += w15 * z4;
         A4[i+15] = a4;
         a5 += w15 * z5;
         A5[i+15] = a5;
         a6 += w15 * z6;
         A6[i+15] = a6;
         a7 += w15 * z7;
         A7[i+15] = a7;
         a8 += w15 * z8;
         A8[i+15] = a8;
         a9 += w15 * z9;
         A9[i+15] = a9;
         a10 += w15 * z10;
         A10[i+15] = a10;
         a11 += w15 * z11;
         A11[i+15] = a11;
         a12 += w15 * z12;
         A12[i+15] = a12;
         a13 += w15 * z13;
         A13[i+15] = a13;
         a14 += w15 * z14;
         A14[i+15] = a14;
         a15 += w15 * z15;
         A15[i+15] = a15;
         a16 += w15 * z16;
         A16[i+15] = a16;
         a17 += w15 * z17;
         A17[i+15] = a17;
         a18 += w15 * z18;
         A18[i+15] = a18;
         a19 += w15 * z19;
         A19[i+15] = a19;

      }
      for (i=M16; i < M; i++)
      {
         const register double x0 = X[i], w0 = W[i];
         register double a0=A0[i], a1=A1[i], a2=A2[i], a3=A3[i], a4=A4[i],
                         a5=A5[i], a6=A6[i], a7=A7[i], a8=A8[i], a9=A9[i],
                         a10=A10[i], a11=A11[i], a12=A12[i], a13=A13[i],
                         a14=A14[i], a15=A15[i], a16=A16[i], a17=A17[i],
                         a18=A18[i], a19=A19[i];

         a0 += x0 * y0;
         a1 += x0 * y1;
         a2 += x0 * y2;
         a3 += x0 * y3;
         a4 += x0 * y4;
         a5 += x0 * y5;
         a6 += x0 * y6;
         a7 += x0 * y7;
         a8 += x0 * y8;
         a9 += x0 * y9;
         a10 += x0 * y10;
         a11 += x0 * y11;
         a12 += x0 * y12;
         a13 += x0 * y13;
         a14 += x0 * y14;
         a15 += x0 * y15;
         a16 += x0 * y16;
         a17 += x0 * y17;
         a18 += x0 * y18;
         a19 += x0 * y19;
         a0 += w0 * z0;
         A0[i] = a0;
         a1 += w0 * z1;
         A1[i] = a1;
         a2 += w0 * z2;
         A2[i] = a2;
         a3 += w0 * z3;
         A3[i] = a3;
         a4 += w0 * z4;
         A4[i] = a4;
         a5 += w0 * z5;
         A5[i] = a5;
         a6 += w0 * z6;
         A6[i] = a6;
         a7 += w0 * z7;
         A7[i] = a7;
         a8 += w0 * z8;
         A8[i] = a8;
         a9 += w0 * z9;
         A9[i] = a9;
         a10 += w0 * z10;
         A10[i] = a10;
         a11 += w0 * z11;
         A11[i] = a11;
         a12 += w0 * z12;
         A12[i] = a12;
         a13 += w0 * z13;
         A13[i] = a13;
         a14 += w0 * z14;
         A14[i] = a14;
         a15 += w0 * z15;
         A15[i] = a15;
         a16 += w0 * z16;
         A16[i] = a16;
         a17 += w0 * z17;
         A17[i] = a17;
         a18 += w0 * z18;
         A18[i] = a18;
         a19 += w0 * z19;
         A19[i] = a19;
      }
      Z += 20;
   }
/*
 * Do remaining columns with NU=1 cleanup
 */
   for (j=N-N20; j; j--, A += lda, Y++)
   {
      double *A0=A;
      const register double y0=Y[0], z0=Z[0];
      ATL_INT i;
      prefY(Y+1+1-1);
      prefY(Z+1+1-1);
      for (i=0; i < M16; i += 16)
      {
         register double a0;
         const double x0=X[i+0], w0=W[i+0], x1=X[i+1], w1=W[i+1], x2=X[i+2],
                      w2=W[i+2], x3=X[i+3], w3=W[i+3], x4=X[i+4], w4=W[i+4],
                      x5=X[i+5], w5=W[i+5], x6=X[i+6], w6=W[i+6], x7=X[i+7],
                      w7=W[i+7], x8=X[i+8], w8=W[i+8], x9=X[i+9], w9=W[i+9],
                      x10=X[i+10], w10=W[i+10], x11=X[i+11], w11=W[i+11],
                      x12=X[i+12], w12=W[i+12], x13=X[i+13], w13=W[i+13],
                      x14=X[i+14], w14=W[i+14], x15=X[i+15], w15=W[i+15];

         prefX(X);
         a0 = A0[i+0];
         a0 += x0 * y0;
         prefX(W);
         a0 += w0 * z0;
         prefA(A0);
         A0[i+0] = a0;
         a0 = A0[i+1];
         a0 += x1 * y0;
         a0 += w1 * z0;
         A0[i+1] = a0;
         a0 = A0[i+2];
         a0 += x2 * y0;
         a0 += w2 * z0;
         A0[i+2] = a0;
         a0 = A0[i+3];
         a0 += x3 * y0;
         a0 += w3 * z0;
         A0[i+3] = a0;
         a0 = A0[i+4];
         a0 += x4 * y0;
         a0 += w4 * z0;
         A0[i+4] = a0;
         a0 = A0[i+5];
         a0 += x5 * y0;
         a0 += w5 * z0;
         A0[i+5] = a0;
         a0 = A0[i+6];
         a0 += x6 * y0;
         a0 += w6 * z0;
         A0[i+6] = a0;
         a0 = A0[i+7];
         a0 += x7 * y0;
         a0 += w7 * z0;
         A0[i+7] = a0;
         a0 = A0[i+8];
         a0 += x8 * y0;
         a0 += w8 * z0;
         prefA(A0);
         A0[i+8] = a0;
         a0 = A0[i+9];
         a0 += x9 * y0;
         a0 += w9 * z0;
         A0[i+9] = a0;
         a0 = A0[i+10];
         a0 += x10 * y0;
         a0 += w10 * z0;
         A0[i+10] = a0;
         a0 = A0[i+11];
         a0 += x11 * y0;
         a0 += w11 * z0;
         A0[i+11] = a0;
         a0 = A0[i+12];
         a0 += x12 * y0;
         a0 += w12 * z0;
         A0[i+12] = a0;
         a0 = A0[i+13];
         a0 += x13 * y0;
         a0 += w13 * z0;
         A0[i+13] = a0;
         a0 = A0[i+14];
         a0 += x14 * y0;
         a0 += w14 * z0;
         A0[i+14] = a0;
         a0 = A0[i+15];
         a0 += x15 * y0;
         a0 += w15 * z0;
         A0[i+15] = a0;

      }
      for (i=M16; i < M; i++)
      {
         const register double x0 = X[i], w0 = W[i];
         register double a0=A0[i];

         a0 += x0 * y0;
         a0 += w0 * z0;
         A0[i] = a0;
      }
      Z++;
   }
}
