#include "atlas_misc.h"
#ifndef PFYDIST
   #define PFYDIST 12
#endif

#if !defined(ATL_3DNow) && !defined(ATL_SSE1) && \
    (PFADIST != 0 || PFXDIST != 0 || PFYDIST != 0)
   #include "atlas_prefetch.h"
#endif

#if defined(ATL_3DNow) || defined(ATL_SSE1)
   #ifndef PFIY
      #define PFIY prefetchnta
   #endif
   #ifndef PFIX
      #define PFIX prefetchnta
   #endif
   #ifndef PFIA
      #ifdef ATL_3DNow
         #define PFIA prefetchw
      #else
         #define PFIA prefetcht0
      #endif
   #endif
#endif
/*
 * X & A are prefetched in M loop PF[A,X]DIST (in bytes) ahead
 */
#if PFADIST == 0                /* flag for no prefetch */
   #define prefA(mem)
#else
   #if defined(ATL_3DNow) || defined(ATL_SSE1)
      #define prefA(mem) __asm__ __volatile__ \
         (Mstr(PFIA) " %0" : : "m" (*(((char *)(mem))+PFADIST)))
   #else
      #if PFLVL == 2
         #define prefA(mem) ATL_pfl2W(((char*)mem)+PFADIST)
      #else
         #define prefA(mem) ATL_pfl1W(((char*)mem)+PFADIST)
      #endif
   #endif
#endif
#if PFXDIST == 0                /* flag for no prefetch */
   #define prefX(mem)
#else
   #if defined(ATL_3DNow) || defined(ATL_SSE1)
      #define prefX(mem) __asm__ __volatile__ \
         (Mstr(PFIX) " %0" : : "m" (*(((char *)(mem))+PFXDIST)))
   #else
      #if PFLVL == 2
         #define prefX(mem) ATL_pfl2R(((char*)mem)+PFXDIST)
      #else
         #define prefX(mem) ATL_pfl1R(((char*)mem)+PFXDIST)
      #endif
   #endif
#endif
/*
 * Y is prefetched in N-loop, and always fetches next NU piece
 */
#if PFYDIST == 0                /* flag for no prefetch */
   #define prefY(mem)
#else
   #if defined(ATL_3DNow) || defined(ATL_SSE1)
      #define prefY(mem) \
         __asm__ __volatile__ (Mstr(PFIY) " %0" : : "m" (*(((char *)(mem)))))
   #else
      #if PFLVL == 2
         #define prefY(mem) ATL_pfl2R(mem)
      #else
         #define prefY(mem) ATL_pfl1R(mem)
      #endif
   #endif
#endif
#ifndef ATL_CINT
   #define ATL_CINT const int
#endif
#ifndef ATL_INT
   #define ATL_INT int
#endif
void ATL_UGERK(ATL_CINT M0, ATL_CINT N, const double *X, const double *Y, 
               double *A, ATL_CINT lda0)
/*
 * 4x12 unrolled r1_c.
 * Extracted from ATLAS/tune/blas/gemv/atlas-l2g.base
 */
{
   ATL_CINT N12=(N/12)*12, M=M0+M0, M4=((M0/4)*4)<<1,
            lda=lda0+lda0, lda12=lda*12;
   ATL_INT j;

   for (j=N12; j; j -= 12, A += lda12, Y += 24)
   {
      double *A0=A, *A1=A0+lda, *A2=A1+lda, *A3=A2+lda, *A4=A3+lda, *A5=A4+lda,
             *A6=A5+lda, *A7=A6+lda, *A8=A7+lda, *A9=A8+lda, *A10=A9+lda,
             *A11=A10+lda;
      const register double ry0=Y[0], iy0=Y[1], ry1=Y[2], iy1=Y[3], ry2=Y[4],
                            iy2=Y[5], ry3=Y[6], iy3=Y[7], ry4=Y[8], iy4=Y[9],
                            ry5=Y[10], iy5=Y[11], ry6=Y[12], iy6=Y[13],
                            ry7=Y[14], iy7=Y[15], ry8=Y[16], iy8=Y[17],
                            ry9=Y[18], iy9=Y[19], ry10=Y[20], iy10=Y[21],
                            ry11=Y[22], iy11=Y[23];
      ATL_INT i;
      prefY(Y+24+24-1);
      for (i=0; i < M4; i += 8)
      {
         const register double rx0=X[i+0], ix0=X[i+1], rx1=X[i+2], ix1=X[i+3],
                               rx2=X[i+4], ix2=X[i+5], rx3=X[i+6], ix3=X[i+7];

         prefX(X);
         A0[i+0] += ry0*rx0 - iy0*ix0;
         prefA(A0);
         A0[i+0+1] += ry0*ix0 + iy0*rx0;
         A1[i+0] += ry1*rx0 - iy1*ix0;
         prefA(A1);
         A1[i+0+1] += ry1*ix0 + iy1*rx0;
         A2[i+0] += ry2*rx0 - iy2*ix0;
         prefA(A2);
         A2[i+0+1] += ry2*ix0 + iy2*rx0;
         A3[i+0] += ry3*rx0 - iy3*ix0;
         prefA(A3);
         A3[i+0+1] += ry3*ix0 + iy3*rx0;
         A4[i+0] += ry4*rx0 - iy4*ix0;
         prefA(A4);
         A4[i+0+1] += ry4*ix0 + iy4*rx0;
         A5[i+0] += ry5*rx0 - iy5*ix0;
         prefA(A5);
         A5[i+0+1] += ry5*ix0 + iy5*rx0;
         A6[i+0] += ry6*rx0 - iy6*ix0;
         prefA(A6);
         A6[i+0+1] += ry6*ix0 + iy6*rx0;
         A7[i+0] += ry7*rx0 - iy7*ix0;
         prefA(A7);
         A7[i+0+1] += ry7*ix0 + iy7*rx0;
         A8[i+0] += ry8*rx0 - iy8*ix0;
         prefA(A8);
         A8[i+0+1] += ry8*ix0 + iy8*rx0;
         A9[i+0] += ry9*rx0 - iy9*ix0;
         prefA(A9);
         A9[i+0+1] += ry9*ix0 + iy9*rx0;
         A10[i+0] += ry10*rx0 - iy10*ix0;
         prefA(A10);
         A10[i+0+1] += ry10*ix0 + iy10*rx0;
         A11[i+0] += ry11*rx0 - iy11*ix0;
         prefA(A11);
         A11[i+0+1] += ry11*ix0 + iy11*rx0;
         A0[i+2] += ry0*rx1 - iy0*ix1;
         A0[i+2+1] += ry0*ix1 + iy0*rx1;
         A1[i+2] += ry1*rx1 - iy1*ix1;
         A1[i+2+1] += ry1*ix1 + iy1*rx1;
         A2[i+2] += ry2*rx1 - iy2*ix1;
         A2[i+2+1] += ry2*ix1 + iy2*rx1;
         A3[i+2] += ry3*rx1 - iy3*ix1;
         A3[i+2+1] += ry3*ix1 + iy3*rx1;
         A4[i+2] += ry4*rx1 - iy4*ix1;
         A4[i+2+1] += ry4*ix1 + iy4*rx1;
         A5[i+2] += ry5*rx1 - iy5*ix1;
         A5[i+2+1] += ry5*ix1 + iy5*rx1;
         A6[i+2] += ry6*rx1 - iy6*ix1;
         A6[i+2+1] += ry6*ix1 + iy6*rx1;
         A7[i+2] += ry7*rx1 - iy7*ix1;
         A7[i+2+1] += ry7*ix1 + iy7*rx1;
         A8[i+2] += ry8*rx1 - iy8*ix1;
         A8[i+2+1] += ry8*ix1 + iy8*rx1;
         A9[i+2] += ry9*rx1 - iy9*ix1;
         A9[i+2+1] += ry9*ix1 + iy9*rx1;
         A10[i+2] += ry10*rx1 - iy10*ix1;
         A10[i+2+1] += ry10*ix1 + iy10*rx1;
         A11[i+2] += ry11*rx1 - iy11*ix1;
         A11[i+2+1] += ry11*ix1 + iy11*rx1;
         A0[i+4] += ry0*rx2 - iy0*ix2;
         A0[i+4+1] += ry0*ix2 + iy0*rx2;
         A1[i+4] += ry1*rx2 - iy1*ix2;
         A1[i+4+1] += ry1*ix2 + iy1*rx2;
         A2[i+4] += ry2*rx2 - iy2*ix2;
         A2[i+4+1] += ry2*ix2 + iy2*rx2;
         A3[i+4] += ry3*rx2 - iy3*ix2;
         A3[i+4+1] += ry3*ix2 + iy3*rx2;
         A4[i+4] += ry4*rx2 - iy4*ix2;
         A4[i+4+1] += ry4*ix2 + iy4*rx2;
         A5[i+4] += ry5*rx2 - iy5*ix2;
         A5[i+4+1] += ry5*ix2 + iy5*rx2;
         A6[i+4] += ry6*rx2 - iy6*ix2;
         A6[i+4+1] += ry6*ix2 + iy6*rx2;
         A7[i+4] += ry7*rx2 - iy7*ix2;
         A7[i+4+1] += ry7*ix2 + iy7*rx2;
         A8[i+4] += ry8*rx2 - iy8*ix2;
         A8[i+4+1] += ry8*ix2 + iy8*rx2;
         A9[i+4] += ry9*rx2 - iy9*ix2;
         A9[i+4+1] += ry9*ix2 + iy9*rx2;
         A10[i+4] += ry10*rx2 - iy10*ix2;
         A10[i+4+1] += ry10*ix2 + iy10*rx2;
         A11[i+4] += ry11*rx2 - iy11*ix2;
         A11[i+4+1] += ry11*ix2 + iy11*rx2;
         A0[i+6] += ry0*rx3 - iy0*ix3;
         A0[i+6+1] += ry0*ix3 + iy0*rx3;
         A1[i+6] += ry1*rx3 - iy1*ix3;
         A1[i+6+1] += ry1*ix3 + iy1*rx3;
         A2[i+6] += ry2*rx3 - iy2*ix3;
         A2[i+6+1] += ry2*ix3 + iy2*rx3;
         A3[i+6] += ry3*rx3 - iy3*ix3;
         A3[i+6+1] += ry3*ix3 + iy3*rx3;
         A4[i+6] += ry4*rx3 - iy4*ix3;
         A4[i+6+1] += ry4*ix3 + iy4*rx3;
         A5[i+6] += ry5*rx3 - iy5*ix3;
         A5[i+6+1] += ry5*ix3 + iy5*rx3;
         A6[i+6] += ry6*rx3 - iy6*ix3;
         A6[i+6+1] += ry6*ix3 + iy6*rx3;
         A7[i+6] += ry7*rx3 - iy7*ix3;
         A7[i+6+1] += ry7*ix3 + iy7*rx3;
         A8[i+6] += ry8*rx3 - iy8*ix3;
         A8[i+6+1] += ry8*ix3 + iy8*rx3;
         A9[i+6] += ry9*rx3 - iy9*ix3;
         A9[i+6+1] += ry9*ix3 + iy9*rx3;
         A10[i+6] += ry10*rx3 - iy10*ix3;
         A10[i+6+1] += ry10*ix3 + iy10*rx3;
         A11[i+6] += ry11*rx3 - iy11*ix3;
         A11[i+6+1] += ry11*ix3 + iy11*rx3;
      }
      for (i=M4; i < M; i += 2)
      {
         const register double rx0 = X[i], ix0 = X[i+1];
         A0[i] += ry0 * rx0 - iy0 * ix0;
         A0[i+1] += ry0 * ix0 + iy0 * rx0;
         A1[i] += ry1 * rx0 - iy1 * ix0;
         A1[i+1] += ry1 * ix0 + iy1 * rx0;
         A2[i] += ry2 * rx0 - iy2 * ix0;
         A2[i+1] += ry2 * ix0 + iy2 * rx0;
         A3[i] += ry3 * rx0 - iy3 * ix0;
         A3[i+1] += ry3 * ix0 + iy3 * rx0;
         A4[i] += ry4 * rx0 - iy4 * ix0;
         A4[i+1] += ry4 * ix0 + iy4 * rx0;
         A5[i] += ry5 * rx0 - iy5 * ix0;
         A5[i+1] += ry5 * ix0 + iy5 * rx0;
         A6[i] += ry6 * rx0 - iy6 * ix0;
         A6[i+1] += ry6 * ix0 + iy6 * rx0;
         A7[i] += ry7 * rx0 - iy7 * ix0;
         A7[i+1] += ry7 * ix0 + iy7 * rx0;
         A8[i] += ry8 * rx0 - iy8 * ix0;
         A8[i+1] += ry8 * ix0 + iy8 * rx0;
         A9[i] += ry9 * rx0 - iy9 * ix0;
         A9[i+1] += ry9 * ix0 + iy9 * rx0;
         A10[i] += ry10 * rx0 - iy10 * ix0;
         A10[i+1] += ry10 * ix0 + iy10 * rx0;
         A11[i] += ry11 * rx0 - iy11 * ix0;
         A11[i+1] += ry11 * ix0 + iy11 * rx0;
      }
   }
/*
 * Do remaining columns with NU=1 cleanup
 */
   for (j=N-N12; j; j--, A += lda, Y += 2)
   {
      double *A0=A;
      const register double ry0=Y[0], iy0=Y[1];
      ATL_INT i;
      prefY(Y+24+24-1);
      for (i=0; i < M4; i += 8)
      {
         const register double rx0=X[i+0], ix0=X[i+1], rx1=X[i+2], ix1=X[i+3],
                               rx2=X[i+4], ix2=X[i+5], rx3=X[i+6], ix3=X[i+7];

         prefX(X);
         A0[i+0] += ry0*rx0 - iy0*ix0;
         prefA(A0);
         A0[i+0+1] += ry0*ix0 + iy0*rx0;
         A0[i+2] += ry0*rx1 - iy0*ix1;
         A0[i+2+1] += ry0*ix1 + iy0*rx1;
         A0[i+4] += ry0*rx2 - iy0*ix2;
         A0[i+4+1] += ry0*ix2 + iy0*rx2;
         A0[i+6] += ry0*rx3 - iy0*ix3;
         A0[i+6+1] += ry0*ix3 + iy0*rx3;
      }
      for (i=M4; i < M; i += 2)
      {
         const register double rx0 = X[i], ix0 = X[i+1];
         A0[i] += ry0 * rx0 - iy0 * ix0;
         A0[i+1] += ry0 * ix0 + iy0 * rx0;
      }
   }
}
